<?php

if (!defined('ABSPATH')) die;

class v4oAuthUser
{
    /** @var WP_User|WP_Error $user */
    private $user = false;
    public $me = [];
    /** @var v4oAuthIdentity $v4oAuthIdentity */
    private $v4oAuthIdentity;

    /**
     * v4oAuthUser constructor.
     * @param false|v4oAuthIdentity $v4oAuthIdentity
     * @throws Exception
     */
    public function __construct($v4oAuthIdentity)
    {
        if (empty($v4oAuthIdentity))
            return;

        $this->v4oAuthIdentity = $v4oAuthIdentity;

        $me = $v4oAuthIdentity->api_me();
        $this->me = $me;

        $this->match();
    }

    /**
     * login (or register and login) a wordpress user based on their oauth identity:
     *
     * @throws Exception
     */
    public function login()
    {
        if ($this->user === false || is_wp_error($this->user))
            $this->create();

        // check if the user was actually created:
        if (is_wp_error($this->user)) {
            // there was an error during registration, redirect and notify the user:
			error_log('v4oAuthUser error: ' . $this->user->get_error_message());
			$this->error(__('There was an error while logging in with your user. Please try again or contact the administrator.', 'v4-oauth'));
        } else {

            $this->update();

            if (
                is_user_logged_in() &&
                $this->user->ID != get_current_user_id()
            ) {
                wp_logout();
            }

            if (!is_user_logged_in()) {

                wp_clear_auth_cookie();
                wp_set_current_user($this->user->ID);
                wp_set_auth_cookie($this->user->ID);

                if (get_current_user_id() === $this->user->ID) {
                    $_SESSION['V4OA']['RESULT'] = __('You are logged in!', 'v4-oauth');
                    do_action('wp_login', $this->user->user_login, $this->user);
                } else
                    $this->error(__('Something went wrong. Please try again or contact the administrator.', 'v4-oauth'));
            }

        }
    }

    /**
     * match the oauth identity to an existing wordpress user account
     *
     * @throws Exception
     */
    public function match()
    {
        // attempt to get a wordpress user id from the database that matches the $oauth_identity['id'] value:
        global $wpdb;
        $usermeta_table = $wpdb->usermeta;
        $query_string = "SELECT $usermeta_table.user_id FROM $usermeta_table WHERE $usermeta_table.meta_key = 'crm_id' AND $usermeta_table.meta_value = '{$this->me['id']}'";
		$user_id = $wpdb->get_var($query_string);

		// fallback old wpoa_identity
		if (empty($user_id)) {
			$query_string = "SELECT $usermeta_table.user_id FROM $usermeta_table WHERE $usermeta_table.meta_key = 'wpoa_identity' AND $usermeta_table.meta_value LIKE '%|{$this->me['id']}|%'";
			$user_id = $wpdb->get_var($query_string);
		}

		// fallback client_id and email
		if (empty($user_id)) {
			$user_id = $this->me['client_id'];
		}

        // attempt to get a wordpress user with the matched id:
        $user = get_user_by('id', $user_id);

        if (is_a($user, 'WP_User')) {
			// check email match
			if($user->user_email !== $this->me['email'])
				$this->error(__('User is incorrect. Please contact the administrator.', 'v4-oauth'));
			// check if trying to override an admin
			foreach ($user->roles as $user_role) {
				$role = get_role($user_role);
				if ($role && $role->has_cap('manage_options'))
					$this->error(__('Sorry. Login as user who can manage_options via 1CRM is forbidden.', 'v4-oauth'));
			}
		}else if (email_exists($this->me['email']))
			$this->error(__('E-Mail already exists.', 'v4-oauth'));

        $this->user = $user;
    }

    /**
     * @throws Exception
     */
    private function create()
    {
        $this->verify_me();

//        $username = uniqid('', true);
        $username = wp_generate_uuid4();
        $password = wp_generate_password();
        $email = $this->me ? $this->me['email'] : '';

        // now attempt to generate the user and get the user id:
        $user_id = wp_create_user($username, $password, $email); // we use wp_create_user instead of wp_insert_user so we can handle the error when the user being registered already exists

        // check if the user was actually created:
        if (is_wp_error($user_id)) {
            // there was an error during registration, redirect and notify the user:
			error_log('v4oAuthUser error: ' . $this->user->get_error_message());
			$this->error(__('There was an error while creating your user. Please try again or contact the administrator.', 'v4-oauth'));
        }

        add_user_meta($user_id, 'wpoa_identity', '1CRM|' . $this->me['id'] . '|' . time());
        add_user_meta($user_id, 'crm_id', $this->me['id']);

        $user = get_user_by('id', $user_id);
        $this->user = $user;
    }

    /**
     * update user
     *
     * @return bool
     * @throws Exception
     */
    public function update()
    {
        $this->verify_me();

        // check if the user was actually created
        if (is_wp_error($this->user)) {
            // there was an error during registration, redirect and notify the user:
			error_log('v4oAuthUser error: ' . $this->user->get_error_message());
            $this->error(__('There was an error while updating your user. Please try again or contact the administrator.', 'v4-oauth'));
        }

        global $wpdb;
        $user_id = $this->user->ID;
        $username = "user" . $user_id;

        /*
         * update user data
         */

        //roles
        if (!empty($this->me['role']) && get_role($this->me['role']))
            $role = $this->me['role'];
        else
            $role = 'subscriber';

        //name
        $real_name = $this->me['name'];
        $user_nice_name = $display_name = $real_name != "" ? $real_name : $username;
        /**
         * IMPORTANT: triggered 'logout_on_delivery_failure' (because of wp_user_update)
         */
        $result = wp_update_user(array(
            'ID' => $user_id,
            'role' => $role,
            'user_nicename' => $user_nice_name,
            'display_name' => $display_name,
            'user_email' => $this->me['email']
        ));

        //additional roles
        $user = new WP_User($result);
        if (!empty($this->me['additional_roles']))
            foreach ($this->me['additional_roles'] as $additional_role)
                if (get_role($additional_role))
                    $user->add_role($additional_role);

//        $result['update_username_result'] = $wpdb->update($wpdb->users, array('user_login' => $username), array('ID' => $user_id));


        /*
         * update user meta data
         */
        update_user_meta($user_id, 'nickname', $user_nice_name);
        if(isset($this->me['first_name']))
        	update_user_meta($user_id, 'first_name', $this->me['first_name']);
        if(isset($this->me['last_name']))
        	update_user_meta($user_id, 'last_name', $this->me['last_name']);
        if(isset($this->me['id']))
        	update_user_meta($user_id, 'crm_id', $this->me['id']);
        if(isset($this->me['birthdate']))
        	update_user_meta($user_id, 'billing_birthday', $this->me['birthdate']);
        if(isset($this->me['first_name']))
        	update_user_meta($user_id, 'billing_first_name', $this->me['first_name']);
        if(isset($this->me['last_name']))
        	update_user_meta($user_id, 'billing_last_name', $this->me['last_name']);


		if(isset($this->me['billing_address']['street'])) {
			$billing_address = explode("\n", $this->me['billing_address']['street'], 2);
			update_user_meta($user_id, 'billing_address_1', $billing_address[0]);
			if (isset($billing_address[1]))
				update_user_meta($user_id, 'billing_address_2', $billing_address[1]);
		}
        if(isset($this->me['billing_address']['city']))
        	update_user_meta($user_id, 'billing_city', $this->me['billing_address']['city']);
        if(isset($this->me['billing_address']['state']))
        	update_user_meta($user_id, 'billing_state', $this->me['billing_address']['state']);
        if(isset($this->me['billing_address']['postalcode']))
        	update_user_meta($user_id, 'billing_postcode', $this->me['billing_address']['postalcode']);
        if(isset($this->me['billing_address']['country']))
        	update_user_meta($user_id, 'billing_country', $this->me['billing_address']['country']);
        if(isset($this->me['billing_address']['company']))
        	update_user_meta($user_id, 'billing_company', $this->me['billing_address']['company']);
        if(isset($this->me['billing_address']['contact']))
        	update_user_meta($user_id, 'billing_contact', $this->me['billing_address']['contact']);
        if(isset($this->me['email']))
        	update_user_meta($user_id, 'billing_email', $this->me['email']);
        if(isset($this->me['salutation']))
        	update_user_meta($user_id, 'billing_title', $this->me['salutation']);
        if(isset($this->me['salutation']))
        	update_user_meta($user_id, 'billing_salutation', $this->me['salutation']);
        if(isset($this->me['phone']))
        	update_user_meta($user_id, 'billing_phone', str_replace('/', ' ', $this->me['phone']));

		if(isset($this->me['shipping_address']['contact'])) {
			$shipping_name_array = explode(" ", $this->me['shipping_address']['contact'], 2);
			update_user_meta($user_id, 'shipping_first_name', $shipping_name_array[0]);
			if (isset($shipping_name_array[1]))
				update_user_meta($user_id, 'shipping_last_name', $shipping_name_array[1]);
		}

		if(isset($this->me['shipping_address']['street'])) {
			$shipping_address = explode("\n", $this->me['shipping_address']['street'], 2);
			update_user_meta($user_id, 'shipping_address_1', $shipping_address[0]);
			if (isset($shipping_address[1]))
				update_user_meta($user_id, 'shipping_address_2', $shipping_address[1]);
		}

        if(isset($this->me['shipping_address']['city']))
        	update_user_meta($user_id, 'shipping_city', $this->me['shipping_address']['city']);
        if(isset($this->me['shipping_address']['state']))
        	update_user_meta($user_id, 'shipping_state', $this->me['shipping_address']['state']);
        if(isset($this->me['shipping_address']['postalcode']))
        	update_user_meta($user_id, 'shipping_postcode', $this->me['shipping_address']['postalcode']);
        if(isset($this->me['shipping_address']['country']))
        	update_user_meta($user_id, 'shipping_country', $this->me['shipping_address']['country']);
        if(isset($this->me['shipping_address']['company']))
        	update_user_meta($user_id, 'shipping_company', $this->me['shipping_address']['company']);
        if(isset($this->me['shipping_address']['contact']))
        	update_user_meta($user_id, 'shipping_contact', $this->me['shipping_address']['contact']);

        /* events manager */
		if(isset($billing_address)) {
			update_user_meta($user_id, 'dbem_address', $billing_address[0]);
			if (isset($billing_address[1]))
				update_user_meta($user_id, 'dbem_address_2', $billing_address[1]);
		}
        if(isset($this->me['billing_address']['city']))
        	update_user_meta($user_id, 'dbem_city', $this->me['billing_address']['city']);
        if(isset($this->me['billing_address']['state']))
        	update_user_meta($user_id, 'dbem_state', $this->me['billing_address']['state']);
        if(isset($this->me['billing_address']['postalcode']))
        	update_user_meta($user_id, 'dbem_zip', $this->me['billing_address']['postalcode']);
        if(isset($this->me['billing_address']['country']))
        	update_user_meta($user_id, 'dbem_country', $this->me['billing_address']['country']);
        if(isset($this->me['billing_address']['phone']))
        	update_user_meta($user_id, 'dbem_phone', $this->me['billing_address']['phone']);
        if(isset($this->me['billing_address']['fax']))
        	update_user_meta($user_id, 'dbem_fax', $this->me['billing_address']['fax']);
        if(isset($this->me['billing_address']['company']))
        	update_user_meta($user_id, 'dbem_company', $this->me['billing_address']['company']);


        /* meta_data */
		if(!empty($this->me['meta_data']))
			foreach ($this->me['meta_data'] as $k => $v)
				update_user_meta($user_id, $k, $v);


        //WooCommerce Customer update
		//TODO ? WC()->customer = new WC_Customer($this->user->ID);
        if (function_exists('WC') && !empty(WC()->customer)) {
        	if(isset($this->me['first_name']))
            	WC()->customer->set_first_name($this->me['first_name']);
            if(isset($this->me['last_name']))
            	WC()->customer->set_last_name($this->me['last_name']);
            if(isset($this->me['billing_address']['company']))
            	WC()->customer->set_billing_company($this->me['billing_address']['company']);
            if(isset($this->me['billing_address']['country']))
            	WC()->customer->set_billing_country($this->me['billing_address']['country']);
			if(isset($billing_address[0]))
            	WC()->customer->set_billing_address_1($billing_address[0]);
            if(isset($billing_address[1]))
            	WC()->customer->set_billing_address_2($billing_address[1]);
            if(isset($this->me['billing_address']['city']))
            	WC()->customer->set_billing_city($this->me['billing_address']['city']);
            if(isset($this->me['billing_address']['state']))
            	WC()->customer->set_billing_state($this->me['billing_address']['state']);
            if(isset($this->me['billing_address']['postalcode']))
            	WC()->customer->set_billing_postcode($this->me['billing_address']['postalcode']);
            if(isset($this->me['billing_address']['phone']))
            	WC()->customer->set_billing_phone($this->me['billing_address']['phone']);
            if(isset($this->me['email']))
            	WC()->customer->set_email($this->me['email']);
        }

        $user = get_user_by('id', $user_id);
        $this->user = $user;

		//if id is unknown to 1CRM, send it to 1CRM
		if(empty($this->me['client_id']) && class_exists('v4\wp\plugins\wp1crm\Customer')) {
			$customer = new v4\wp\plugins\wp1crm\Customer($user_id);
			$customer->send();
		}

        return $result;
    }

    /**
     * throw error if me is not verified
     *
     * @throws Exception
     */
    private function verify_me()
    {
        // check if trying to login with a user who can manage_options
        $role = get_role($this->me['role']);
        if ($role && $role->has_cap('manage_options')) {
            $this->error(__('Sorry, you are not allowed to override that user.', 'v4-oauth'));
        }
    }

    /**
     * @param $msg
     * @param int $code
     * @throws Exception
     */
    private function error($msg, $code = 400)
    {
        throw new Exception($msg, $code);
    }
}